--  ===============================
--	Tabard Addict - TabardAddict.lua
--	By: gmz323(Greg)
--  ===============================

-- Constants
local TABARD_ADDICT_VERSION = "1.08"
local TABARD_ADDICT_DATA_ROWS = 87

-- Locals
local TabardAddictServerData = {} -- equip data from server
local TabardAddictDoServerQuery;

local CurrentTabardEntry_line = 0;
local CurrentTabardEntries_itemID = {} 
local CurrentTabardEntries_itemLink = {}

local tempDisplayData = {} -- temp table built to show subset of data
local tempDisplayData_rowcount;
local tabardsEquipped = 0;

-- Slash commands
SLASH_TABARDADDICT1 = "/ta"
SLASH_TABARDADDICT2 = "/tabardaddict"

SlashCmdList["TABARDADDICT"] = function(msg, editbox)
	if frameTabardAddict:IsShown() then
		frameTabardAddict:Hide()
	else
		frameTabardAddict:Show()
	end
end

-- Debugging
local verboseMode = false;

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_PopulateDisplayTable()
	
	local maintableRow, j, b1, b2, displayThisRow;
	
	wipe(tempDisplayData);
	
	tempDisplayData_rowcount = 0;
	
	-- This creates a temp table by joining local tabard data and server results
	for maintableRow in ipairs(taTabardData)
	do
		-- determine if this row should be displayed -- by Tabard Type
		if ((taTabardData[maintableRow][5] == 0) and (TabardAddictConfigCB_Other == 1)) then
			displayThisRow = 1;
		elseif ((taTabardData[maintableRow][5] == 1) and (TabardAddictConfigCB_PVP == 1)) then
			displayThisRow = 1; 	
		elseif ((taTabardData[maintableRow][5] == 2) and (TabardAddictConfigCB_Other == 1)) then
			displayThisRow = 1; 	
		elseif ((taTabardData[maintableRow][5] == 3) and (TabardAddictConfigCB_TCG == 1)) then
			displayThisRow = 1; 
		elseif ((taTabardData[maintableRow][5] == 4) and (TabardAddictConfigCB_NA == 1)) then
			displayThisRow = 1; 
		elseif ((taTabardData[maintableRow][5] == 6) and (TabardAddictConfigCB_BCF == 1)) then
			displayThisRow = 1; 
		elseif ((taTabardData[maintableRow][5] == 7) and (TabardAddictConfigCB_WotLKF == 1)) then
			displayThisRow = 1; 
		elseif ((taTabardData[maintableRow][5] == 8) and (TabardAddictConfigCB_ATF == 1)) then
			displayThisRow = 1; 
		elseif ((taTabardData[maintableRow][5] == 9) and (TabardAddictConfigCB_CF == 1)) then
			displayThisRow = 1; 	
		else
			displayThisRow = 0;
		end;
		
		-- determine if this row should be filtered -- Filter out by Alliance/Horde
		if ((taTabardData[maintableRow][6] == 1) and (TabardAddictConfigCB_AO == 0)) then
			displayThisRow = 0;
		elseif ((taTabardData[maintableRow][6] == 2) and (TabardAddictConfigCB_HO == 0)) then
			displayThisRow = 0; 
		end;
		
		if (displayThisRow == 1) then
			tempDisplayData_rowcount = tempDisplayData_rowcount + 1;
			tempDisplayData[tempDisplayData_rowcount] = {};     -- create a new row
			for j=1,4 do -- copy first 4 column
				tempDisplayData[tempDisplayData_rowcount][j] = taTabardData[maintableRow][j];
			end
			-- column 6 - tabard type
			tempDisplayData[tempDisplayData_rowcount][6] = taTabardData[maintableRow][5];
			-- column 7 - alliance or horde indicator
			tempDisplayData[tempDisplayData_rowcount][7] = taTabardData[maintableRow][6];
		  
			-- init equipped bool
			tempDisplayData[tempDisplayData_rowcount][5] = false;
			
			-- set equipped bool from server data
			for b1,b2 in ipairs(TabardAddictServerData)
			do
				if (b2[1] == tempDisplayData[tempDisplayData_rowcount][1]) then
					tempDisplayData[tempDisplayData_rowcount][5] = TabardAddictServerData[b1][2];
				end
			end
		end
    end
end

--*****************************************************************************
--*****************************************************************************
local function TabardAddict_SortDisplayTable()

	-- currently sorting only on tabard name
	table.sort(tempDisplayData, function(a,b) return a[2]<b[2] end);
	
end

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnEvent(self, event, ...)
	
	-- Check if player changed tabard
	if (event == "PLAYER_EQUIPMENT_CHANGED") then
		
		local slot, hasItem = ...;
		if hasItem == nil then 
			hasItem = 0;
		end;
		
		-- addon is shown - do an update and return to tab1
		if frameTabardAddict:IsShown() then	
			-- if tabard slot and item put into slot - refresh listing and count
			if ( (slot == INVSLOT_TABARD) and (hasItem == 1) ) then
				TabardAddictDoServerQuery = true;
				TabardAddict_OnShow(self);
			end;
			if (verboseMode) then
				print("Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. hasItem);
			end;
		else
			-- addon not shown - updated on next show event
			if ( (slot == INVSLOT_TABARD) and (hasItem == 1) ) then
				TabardAddictDoServerQuery = true;
			end;
			if (verboseMode) then
				print("Not Shown - Registered Event = " .. event .. "; slot = " .. slot .. " hasItem = " .. hasItem);
			end;
		end;
	end;
end
 
--*****************************************************************************
--*****************************************************************************
function TabardAddict_GetServerData() 

	local j,m,r,a,b;
	
	wipe(TabardAddictServerData);
	
	-- id 1020 = ten tabards achievement
	a=GetAchievementCriteriaInfo 
	for b=1,TABARD_ADDICT_DATA_ROWS
	do 
		_,_,_,_,_,_,_,_,_,j = a(1020,b); 
		_,_,m,_,_,_,_,r,_,_ = a(j);
		TabardAddictServerData[b] = {}     -- create a new row
		TabardAddictServerData[b][1] = r;  -- save id
		TabardAddictServerData[b][2] = m;  -- save "criteria met" true/false
		if m==true then
			-- increment total tabards equipped
			tabardsEquipped=tabardsEquipped + 1;
		end;
	end
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictScrollBar_Update()
	local line; -- 1 through 8 of our window to scroll
	local lineplusoffset; -- an index into our data calculated from the scroll offset
	
	FauxScrollFrame_Update(TabardAddictScrollBar,tempDisplayData_rowcount,8,40);
	for line=1,8 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(TabardAddictScrollBar);
		
		-- set alpha values for each row
		getglobal("TabardAddictEntry"..line).TabardAddictButtonBack:SetAlpha(0.35);
		getglobal("TabardAddictEntry"..line).TabardAddictButtonHighlight:SetAlpha(0.70);
		
		if lineplusoffset <= tempDisplayData_rowcount then
			getglobal("TabardAddictEntry"..line).TabardName:SetText(tempDisplayData[lineplusoffset][2]);
			getglobal("TabardAddictEntry"..line).TabardName:Show();
			
			-- check if equipped
			if (tempDisplayData[lineplusoffset][5] == true) then
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText("Been Equipped: |cff00FF00Yes|r");
				-- show overlay on equipped tabards
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Show(); 
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:SetDrawLayer("OVERLAY",5);
			else
				getglobal("TabardAddictEntry"..line).TabardEquipped:SetText("Been Equipped: |cffFF0000No|r");
				getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide();
			end
			getglobal("TabardAddictEntry"..line).TabardEquipped:Show();
			
			-- set tabard icon texture
			getglobal("TabardAddictEntry"..line).TabardIcon:SetTexture(tempDisplayData[lineplusoffset][4]);
			getglobal("TabardAddictEntry"..line).TabardIcon:Show();
			
			-- set local vars
			CurrentTabardEntries_itemID[line] = tempDisplayData[lineplusoffset][1];
			CurrentTabardEntries_itemLink[line] = tempDisplayData[lineplusoffset][3];
			
			-- set faction icon
			if (tempDisplayData[lineplusoffset][7] == 2) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Horde.png");
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			elseif (tempDisplayData[lineplusoffset][7] == 1) then
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetTexture("Interface\\BattlefieldFrame\\Battleground-Alliance.png");
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Show();
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:SetDrawLayer("OVERLAY",7);
			else
				getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide();
			end;
			
			-- update tooltip
			if (CurrentTabardEntry_line > 0) then
				TabardAddictEntryButton_OnEnter(CurrentTabardEntry_line);
			end
		else
			getglobal("TabardAddictEntry"..line).TabardName:Hide();
			getglobal("TabardAddictEntry"..line).TabardEquipped:Hide();
			getglobal("TabardAddictEntry"..line).TabardIcon:Hide();
			getglobal("TabardAddictEntry"..line).TabardIconOverlay:Hide();
			getglobal("TabardAddictEntry"..line).TabardFactionIcon:Hide();
			-- set local vars
			CurrentTabardEntries_itemID[line] = 0;
			CurrentTabardEntries_itemLink[line] = 0;
			
			-- warn user
			if ((tempDisplayData_rowcount == 0) and (line == 2)) then
				getglobal("TabardAddictEntry"..line).TabardName:SetText("No Tabards to show - Check Config!");
				getglobal("TabardAddictEntry"..line).TabardName:Show();
			end;
		end
	end	
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnEnter(line)
	CurrentTabardEntry_line = line;
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnLeave()
	CurrentTabardEntry_line = 0;
	ResetCursor();
	GameTooltip:Hide();
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnUpdate(line)
	
	local itemID = CurrentTabardEntries_itemID[line];
	
	-- Show tooltip if over tabard icon
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		GameTooltip:SetOwner(frameTabardAddict, "ANCHOR_RIGHT", 0, -120);
		GameTooltip:SetHyperlink("item:"..itemID);
		GameTooltip:Show();
		-- if ctrl is down show inspect cursor else reset it
		if ( IsModifiedClick("DRESSUP") ) then
			ShowInspectCursor();
		else
			ResetCursor();
		end
	else
		-- not over icon - hide tooltip and reset cursor
		GameTooltip:Hide();
		ResetCursor();
	end
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictEntryButton_OnModifiedClick(line)
	local itemID = CurrentTabardEntries_itemID[line];
	local itemLink = CurrentTabardEntries_itemLink[line];
	
	-- If there was a DRESSUP or CHATLINK click over a tabard icon handle it
	if ( (MouseIsOver(getglobal("TabardAddictEntry"..line).TabardIcon)) and (itemID > 0) ) then
		if ( IsModifiedClick("DRESSUP") ) then
			DressUpItemLink(itemID);
		elseif ( IsModifiedClick("CHATLINK") ) then
			if (ChatEdit_GetActiveWindow()) then
				ChatEdit_InsertLink(itemLink);
			end;
		end;
	end;
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictTab_OnClick(self, tabID)
	if ( not tabID ) then
		tabID = self:GetID();
	end
	PanelTemplates_SetTab(frameTabardAddict, tabID);
	if ( tabID == 1 ) then
		-- Tabards tab clicked
		TabardAddictScrollBar:Show();
		TabardAddictTab1:Show();
		TabardAddictTab2:Hide();
		TabardAddictTab3:Hide();
		-- populate the display table
		TabardAddict_PopulateDisplayTable();
		-- sort the display table
		TabardAddict_SortDisplayTable();
		-- initial display
		TabardAddictScrollBar_Update();
		TabardAddictShowTabards();
	elseif ( tabID == 2 ) then
		-- Config tab clicked
		TabardAddictScrollBar:Hide();
		TabardAddictTab1:Hide();
		TabardAddictTab2:Show();
		TabardAddictTab3:Hide();
		-- set checkbox values
		CheckButtonAO:SetChecked(TabardAddictConfigCB_AO);
		CheckButtonHO:SetChecked(TabardAddictConfigCB_HO);
		CheckButtonTCG:SetChecked(TabardAddictConfigCB_TCG);
		CheckButtonOther:SetChecked(TabardAddictConfigCB_Other);
		CheckButtonNA:SetChecked(TabardAddictConfigCB_NA);
		CheckButtonPVP:SetChecked(TabardAddictConfigCB_PVP);
		CheckButtonBCF:SetChecked(TabardAddictConfigCB_BCF);
		CheckButtonWotLKF:SetChecked(TabardAddictConfigCB_WotLKF);
		CheckButtonATF:SetChecked(TabardAddictConfigCB_ATF);
		CheckButtonCF:SetChecked(TabardAddictConfigCB_CF);
		TabardAddictHideTabards();
	elseif ( tabID == 3 ) then
		-- About tab clicked
		TabardAddictScrollBar:Hide();
		TabardAddictTab1:Hide();
		TabardAddictTab2:Hide();
		TabardAddictTab3:Show();
		TabardAddictHideTabards();
	end
	PlaySound("igCharacterInfoTab");
end

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnShow(self)
	
	local tabardsNextAchievement = 0;
	
	PlaySound("igCharacterInfoOpen");
	
	-- set description text
	TabardAddictTab1.taDescText:SetText("Many of the tabards listed below may not be available to "..UnitName("player")..". Check config tab for display filtering!");
	
	if (TabardAddictDoServerQuery == true) then
		-- reset # of equipped tabards
		tabardsEquipped = 0;
		-- query server for data
		TabardAddict_GetServerData();
		TabardAddictDoServerQuery = false;
	end;

	-- populate counts
	if (tabardsEquipped >= 30) then
		tabardsNextAchievement = "none";
	elseif (tabardsEquipped >= 25) then
		tabardsNextAchievement = 30;
	elseif (tabardsEquipped >= 10) then
		tabardsNextAchievement = 25;
	elseif (tabardsEquipped >= 1) then
		tabardsNextAchievement = 10;
	else
		tabardsNextAchievement = 1;
	end;
	
	-- populate equipped count and next achievement text
	TabardAddictTab1.taTabardTotalText:SetText("Tabards Equipped: "..tabardsEquipped.."    Next Achievement: "..tabardsNextAchievement);
	
	-- populate the display table
	TabardAddict_PopulateDisplayTable();
	-- sort the display table
	TabardAddict_SortDisplayTable();
	-- initial display
	TabardAddictScrollBar_Update();
	
	-- show/hide panels
	TabardAddictTab1:Show();
	TabardAddictTab2:Hide();
	TabardAddictTab3:Hide();
	PanelTemplates_SetTab(frameTabardAddict, 1);
	TabardAddictShowTabards();
	
end

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnHide(self)

	PlaySound("igCharacterInfoClose");
	-- collectgarbage("collect");
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictShowTabards()
	
	local line;

	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Show();
	end;
end

--*****************************************************************************
--*****************************************************************************
function TabardAddictHideTabards()
	
	local line;

	for line=1,8 do
		getglobal("TabardAddictEntry"..line):Hide();
	end;
end

--*****************************************************************************
--*****************************************************************************
function TabardAddict_SetConfigVariables(self)

	if (CheckButtonAO:GetChecked() == 1) then
		TabardAddictConfigCB_AO = 1;
	else
		TabardAddictConfigCB_AO = 0;
	end;
	if (CheckButtonHO:GetChecked() == 1) then
		TabardAddictConfigCB_HO = 1;
	else
		TabardAddictConfigCB_HO = 0;
	end;
	if (CheckButtonTCG:GetChecked() == 1) then
		TabardAddictConfigCB_TCG = 1;
	else
		TabardAddictConfigCB_TCG = 0;
	end;
	if (CheckButtonOther:GetChecked() == 1) then
		TabardAddictConfigCB_Other = 1;
	else
		TabardAddictConfigCB_Other = 0;
	end;
	if (CheckButtonNA:GetChecked() == 1) then
		TabardAddictConfigCB_NA = 1;
	else
		TabardAddictConfigCB_NA = 0;
	end;
	if (CheckButtonPVP:GetChecked() == 1) then
		TabardAddictConfigCB_PVP = 1;
	else
		TabardAddictConfigCB_PVP = 0;
	end;
	if (CheckButtonBCF:GetChecked() == 1) then
		TabardAddictConfigCB_BCF = 1;
	else
		TabardAddictConfigCB_BCF = 0;
	end;
	if (CheckButtonWotLKF:GetChecked() == 1) then
		TabardAddictConfigCB_WotLKF = 1;
	else
		TabardAddictConfigCB_WotLKF = 0;
	end;
	if (CheckButtonATF:GetChecked() == 1) then
		TabardAddictConfigCB_ATF = 1;
	else
		TabardAddictConfigCB_ATF = 0;
	end;
	if (CheckButtonCF:GetChecked() == 1) then
		TabardAddictConfigCB_CF = 1;
	else
		TabardAddictConfigCB_CF = 0;
	end;
	
end

--*****************************************************************************
--*****************************************************************************
function TabardAddict_OnLoad(self)

	-- Register the frame so it can be dragged
	self:RegisterForDrag("LeftButton");

	-- Register Events
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED");
	
	-- init variables
	TabardAddictDoServerQuery = true;
	
	-- Tab Handling code
	PanelTemplates_SetNumTabs(self, 3);
	PanelTemplates_SetTab(self, 1);
	
	-- show/hide panels
	TabardAddictTab1:Show();
	TabardAddictTab2:Hide();
	TabardAddictTab3:Hide();
	
	-- update portrait emblem
	SetPortraitToTexture(self.taTabardEmblem, "Interface\\ICONS\\INV_Chest_Cloth_30")
	
	self.taTitleText:SetText("Tabard Addict");
	self.taVersionText:SetText("v"..TABARD_ADDICT_VERSION);
	
	-- set description text
	TabardAddictTab2.taConfigDescText:SetText("Tabard Display Configuration");
	TabardAddictTab3.taAboutDescText:SetText("About Tabard Addict");
	TabardAddictAboutInfo:SetText("Tabard Addict is a simple mod that shows which tabards you have and have not equipped on your way to meeting the various tabard achievements. It also allows you to play dressup with tabards not available to your character.|n|nHold down CTRL and left click on a tabard icon to launch the Dressing Room.|n|nUse slash commands |cFFFF0000/ta|r or |cFFFF0000/tabardaddict|r to open and/or close the window.|n|nDownload updates and leave comments at: |cFFFFFF00wowinterface.com|r and |cFFFFFF00wow.curse.com|r|n|nAuthored by: |cFFFFFF00gmz323 (Greg)|r");
	
	-- set alliance or horde background
	local englishFaction = UnitFactionGroup("player");
	if ( englishFaction == "Horde" ) then  -- horde
		TabardAddictBGFaction:Show();
		TabardAddictBGFaction2:Hide();
	else  -- alliance
		TabardAddictBGFaction:Hide();
		TabardAddictBGFaction2:Show();
	end;
	
	-- check for nil global config values
	if (TabardAddictConfigCB_AO == nil) then
		TabardAddictConfigCB_AO = 1;
	end;
	if (TabardAddictConfigCB_HO == nil) then
		TabardAddictConfigCB_HO = 1;
	end;
	if (TabardAddictConfigCB_TCG == nil) then
		TabardAddictConfigCB_TCG = 1;
	end;
	if (TabardAddictConfigCB_Other == nil) then
		TabardAddictConfigCB_Other = 1;
	end;
	if (TabardAddictConfigCB_NA == nil) then
		TabardAddictConfigCB_NA = 1;
	end;
	
	if (TabardAddictConfigCB_PVP == nil) then
		TabardAddictConfigCB_PVP = 1;
	end;
	if (TabardAddictConfigCB_BCF == nil) then
		TabardAddictConfigCB_BCF = 1;
	end;
	if (TabardAddictConfigCB_WotLKF == nil) then
		TabardAddictConfigCB_WotLKF = 1;
	end;
	if (TabardAddictConfigCB_ATF == nil) then
		TabardAddictConfigCB_ATF = 1;
	end;
	if (TabardAddictConfigCB_CF == nil) then
		TabardAddictConfigCB_CF = 1;
	end;
end;



